using System;
using Sirenix.OdinInspector;
using Sirenix.OdinInspector.Editor;
using Sirenix.Utilities;
using Sirenix.Utilities.Editor;
using UnityEditor;
using UnityEngine;

namespace Scripts
{
	public class Item
	{
		[SerializeField] private ItemAsset asset;
		public                   ItemAsset Asset => asset;

		[SerializeField] public bool IsEmpty { get; protected set; }
		[SerializeField] public Guid Guid    { get; protected set; }

		public Item()
		{
			Clear();
		}

		public Item(ItemAsset asset)
		{
			Set(asset);
		}

		public void Set(ItemAsset asset)
		{
			this.asset = asset;
			IsEmpty    = false;
			Guid       = Guid.NewGuid();
		}

		public void Clear()
		{
			asset   = null;
			IsEmpty = true;
			Guid    = Guid.Empty;
		}
	}

#if UNITY_EDITOR

	[DrawerPriority(0, 0, 10000)]
	public class ItemBaseGenericDrawer : OdinValueDrawer<Item>
	{
		private readonly GUIStyle labelStyle = new GUIStyle {richText = true};


		protected override void DrawPropertyLayout(GUIContent label)
		{
			const float SCALE = 0.5f;

			Item item = ValueEntry.SmartValue;

		#region Drawing values

			Rect rect = EditorGUILayout.GetControlRect(false, 150 * SCALE);
			EditorGUI.DrawRect(rect, new Color(0.7f, 0.7f, 0.7f));

		#region Setting to null

			if (SirenixEditorGUI.IconButton(rect.AlignLeft(50 * SCALE).AlignCenterY(50 * SCALE), EditorIcons.X, "Set to null"))
			{
				ValueEntry.SmartValue = null;
			}

		#endregion

			rect = rect.HorizontalPadding(50 * SCALE);

			if (item == null || !item.IsEmpty && item.Asset == null) // No asset/null
			{
				{
					EditorGUI.LabelField(rect.AlignLeft(400 * SCALE).AlignTop(50 * SCALE), $"{label.text}: <color=blue>null</color>", labelStyle);
					ItemAsset newAsset =
						SirenixEditorFields.PreviewObjectField(rect.AlignLeft(100 * SCALE).AlignBottom(100 * SCALE), null as ItemAsset, false, false, false, false);
					if (newAsset != null)
					{
						ValueEntry.SmartValue = newAsset.GetItem();
						Debug.Log("Null item set");
					}
				}
			}
			else // Item drawer
			{
				EditorGUI.LabelField(rect.AlignLeft(400 * SCALE).AlignTop(50 * SCALE),
					$"{label.text}: {(item.IsEmpty ? "<b><color=red>(Empty)</color></b>" : "")} {(item.Asset ? item.Asset.Name : "<i>(No Asset)</i>")} {item.Guid}"
					, labelStyle);

				if (item.IsEmpty) // Empty item
				{
					ItemAsset newAsset =
						SirenixEditorFields.PreviewObjectField(rect.AlignLeft(100 * SCALE).AlignBottom(100 * SCALE), null as ItemAsset, false, false, false, false);
					if (newAsset != null)
					{
						ValueEntry.SmartValue = newAsset.GetItem();
						Debug.Log("Empty item set");
					}
				}
				else
				{
					ItemAsset newAsset = SirenixEditorFields.PreviewObjectField(rect.AlignLeft(100 * SCALE).AlignBottom(100 * SCALE), item.Asset, false, false, false, false);

					if (newAsset == null)
						Property.Tree.DelayActionUntilRepaint(() => item.Clear());
					else if (newAsset != item.Asset)
					{
						ValueEntry.WeakSmartValue = newAsset.GetItem();
						Debug.Log("Existing item set");
					}
				}
			}

		#endregion
		}
	}
#endif
}